@@ -102,16 +102,20 @@ package
 use strict;
 use warnings;
 
-my $orig_fetch_para;
+    # pre process     # post process
+my ($orig_fetch_para, $orig_print_section);
 
 BEGIN {
     $orig_fetch_para = \&fetch_para;
+    $orig_print_section = \&print_section;
     no strict 'refs';
     delete ${__PACKAGE__.'::'}{fetch_para};
+    delete ${__PACKAGE__.'::'}{print_section};
     delete ${__PACKAGE__.'::'}{eval_output_typemap_code};
     delete ${__PACKAGE__.'::'}{eval_input_typemap_code};
 }
 
+# pre process XS function
 sub fetch_para {
     my $self = shift;
     my $ret = $orig_fetch_para->($self, @_);
@@ -157,7 +161,6 @@ sub fetch_para {
         $lines->[0] = $type;
         
         if (!$lines->[1]) {{ # empty sub
-            $DB::single=1;
             my ($class, $func, $var);
             if ($sig =~ /^([^:]+)::([a-zA-Z0-9_\$]+)/) {
                 ($class, $func, $var) = ("$1*", $2, 'THIS');
@@ -170,7 +173,7 @@ sub fetch_para {
                 splice(@$linno, 1, 0, $linno->[0]);
             }
         }}
-        
+                
         if ($lines->[1] and $lines->[1] !~ /^[A-Z]+\s*:/) {
             splice(@$lines, 1, 0, $type =~ /^void(\s|$)/ ? 'PPCODE:' : 'CODE:');
             splice(@$linno, 1, 0, $linno->[0]);
@@ -219,11 +222,113 @@ sub fetch_para {
     return $ret;
 }
 
+# post process XS function
+sub print_section {
+    my $self = shift;
+    my $lines = $self->{line};
+    my $linno = $self->{line_no};
+    
+    # find typemap_in|outcast<>()
+    state $re_parens = qr/(\((?:(?>[^()]+)|(?-1))*\))/;
+    state $re_gtlt = qr/(<(?:(?>[^<>]+)|(?-1))*>)/;
+    my %gen_funcs;
+    foreach my $row (['typemap_incast', 1], ['typemap_outcast', 0]) {
+        my ($kword, $is_input) = @$row;
+        my $re = qr/\b$kword\s*(?<CLASS>$re_gtlt)\s*(?<EXPR>$re_parens)/;
+        foreach my $line (@$lines) {
+            while ($line =~ $re) {
+                my ($class, $expr) = @+{'CLASS', 'EXPR'};
+                $class =~ s/^<//;
+                $class =~ s/>$//;
+                my @args;
+                ($class, @args) = split /\s*,\s*/, $class;
+                for (@args) {
+                    s/^\s+//; s/\s+$//;
+                    die "\e[31m Typemap parameter must have a type at '$_' \e[0m" unless /^(.+?)([a-zA-Z0-9_\$]+)$/;
+                    my ($type, $name) = ($1, $2);
+                    $type =~ s/\s+\*/*/g;
+                    $type =~ s/\s+$//;
+                    $_ = [$type, $name];
+                }
+                
+                my $meth = $is_input ? 'get_inputmap' : 'get_outputmap';
+                my $tmap = $self->{typemap}->$meth(ctype => $class);
+                die "\e[31m No typemap found for '$kword<$class>()', line '$line' \e[0m" unless $tmap;
+                my $subtype = $class; $subtype =~ s/\s*\*$//;
+                my $ntype   = $class; $ntype   =~ s/\s*\*$/Ptr/;
+                my $tmfunc_name = "_${kword}_${ntype}_".join("_", map {"@$_"} @args);
+                $tmfunc_name =~ s/\*/Ptr/g;
+                $tmfunc_name =~ s/\s/_/g;
+                unless ($gen_funcs{$tmfunc_name}) {
+                    my $other = {
+                        var     => 'var',
+                        type    => $class,
+                        subtype => $subtype,
+                        ntype   => $ntype,
+                        arg     => 'arg',
+                    };
+                    if ($is_input) {
+                        $other->{num}          = -1; # not on stack
+                        $other->{init}         = undef;
+                        $other->{printed_name} = 0;
+                        $other->{argoff}       = -1; # not on stack
+                    }
+                    $meth = $is_input ? 'eval_input_typemap_code' : 'eval_output_typemap_code';
+                    my $tmcode = $tmap->code;
+                    my $code = $self->$meth("qq\a$tmcode\a", $other);
+                    my $arg_init = (!$is_input && $tmcode =~ /^\s*\$arg\s*=\s*\NULL\s*;\s*$/m) ? '' : ' = newSV(0)';
+                    my $tmfunc_code = _typemap_inline_func($tmfunc_name, $class, $code, $arg_init, $is_input, \@args, $tmap->{_init_code});
+                    $gen_funcs{$tmfunc_name} = $tmfunc_code;
+                }
+                $line =~ s/$re/${tmfunc_name}::get$expr/;
+            }
+        }
+    }
+    
+    my $gen_code = join "", values %gen_funcs;
+    print $gen_code;
+    
+    return $gen_code.$orig_print_section->($self, @_);
+}
+
+sub _typemap_inline_func {
+    my ($tmfunc_name, $class, $code, $arg_init, $is_input, $args, $tm_init) = @_;
+    $code =~ s/^\s+//s;
+    $code =~ s/\s+$//s;
+    my $additional_args = @$args ? ", ".join(", ", map { "$_->[0] $_->[1]" } @$args) : '';
+    
+    if ($tm_init and @$args) {
+        # if some custom typemap variable is defined and set in typemap INIT section, and is present in $args,
+        # we must remove it to give user a chance to redefine it. Ugly, but there are no other chance with ugly ExtUtils::ParseXS.
+        for (@$args) {
+            my ($type, $name) = @$_;
+            $type =~ s/\*/\\s*\\*/g;
+            $tm_init =~ s/((?:^\s*|;\s*|\}\s*)$type\s+)($name)\b/${1}__pxs_${2}_off__/s;
+        }
+    }
+    $code = "$tm_init\n    $code" if $tm_init;
+    $code =~ s/^[\s;]+$//mg;
+    $code =~ s/\n\n+/\n/g;
+    
+    $code =~ s/^/        /mg;
+    return << "EOF" if $is_input;
+        struct $tmfunc_name { static inline $class get (SV* arg$additional_args) {
+            $class var;\n    $code;
+            return var;
+        }};
+EOF
+    return << "EOF";
+        struct $tmfunc_name { static inline SV* get ($class var$additional_args) {
+            SV* arg$arg_init;\n    $code;
+            return arg;
+        }};
+EOF
+}
+
 sub eval_output_typemap_code {
     my ($self, $code, $other) = @_;
     my ($Package, $ALIAS, $func_name, $Full_func_name, $pname) = @{$self}{qw(Package ALIAS func_name Full_func_name pname)};
     my ($var, $type, $ntype, $subtype, $arg) = @{$other}{qw(var type ntype subtype arg)};
-    my $type_deref = substr($type, -1, 1) eq '*' ? substr($type, 0, length($type)-1) : $type;
     my %p;
   
     no warnings 'uninitialized';
@@ -236,7 +341,6 @@ sub eval_input_typemap_code {
     my ($self, $code, $other) = @_;
     my ($Package, $ALIAS, $func_name, $Full_func_name, $pname) = @{$self}{qw(Package ALIAS func_name Full_func_name pname)};
     my ($var, $type, $num, $init, $printed_name, $arg, $ntype, $argoff, $subtype) = @{$other}{qw(var type num init printed_name arg ntype argoff subtype)};
-    my $type_deref = substr($type, -1, 1) eq '*' ? substr($type, 0, length($type)-1) : $type;
     my %p;
 
     no warnings 'uninitialized';
